@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2007, Jari Sundell
+// Copyright (C) 2005-2011, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -390,4 +390,24 @@ ChunkList::partition_optimize(Queue::iterator first, Queue::iterator last, int w
   return first;
 }
 
+ChunkList::chunk_address_result
+ChunkList::find_address(void* ptr) {
+  iterator first = begin();
+  iterator last = end();
+
+  for (; first != last; first++) {
+    if (!first->is_valid())
+      continue;
+
+    Chunk::iterator partition = first->chunk()->find_address(ptr);
+
+    if (partition != first->chunk()->end())
+      return chunk_address_result(first, partition);
+
+    first++;
+  }
+
+  return chunk_address_result(end(), Chunk::iterator());
+}
+
 }
