@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2007, Jari Sundell
+// Copyright (C) 2005-2011, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -39,10 +39,10 @@
 
 #include <string>
 #include <vector>
-#include <string>
 #include <rak/error_number.h>
 #include <rak/functional.h>
 
+#include "chunk.h"
 #include "chunk_handle.h"
 #include "chunk_list_node.h"
 
@@ -121,6 +121,10 @@ public:
   void                slot_create_chunk(SlotCreateChunk s)     { m_slotCreateChunk = s; }
   void                slot_free_diskspace(SlotFreeDiskspace s) { m_slotFreeDiskspace = s; }
 
+  typedef std::pair<iterator, Chunk::iterator> chunk_address_result;
+
+  chunk_address_result find_address(void* ptr);
+
 private:
   inline bool         is_queued(ChunkListNode* node);
 
