@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2007, Jari Sundell
+// Copyright (C) 2005-2011, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ namespace torrent {
 
 class LIBTORRENT_EXPORT PollEPoll : public torrent::Poll {
 public:
-  typedef std::vector<uint32_t> Table;
+  typedef std::vector<std::pair<uint32_t, Event*> > Table;
 
   static PollEPoll*   create(int maxOpenSockets);
   virtual ~PollEPoll();
@@ -63,6 +63,9 @@ public:
   virtual void        open(torrent::Event* event);
   virtual void        close(torrent::Event* event);
 
+  // torrent::Event::get_fd() was closed outside of our control.
+  virtual void        closed(torrent::Event* event);
+
   // Functions for checking whetever the torrent::Event is listening to r/w/e?
   virtual bool        in_read(torrent::Event* event);
   virtual bool        in_write(torrent::Event* event);
