@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2007, Jari Sundell
+// Copyright (C) 2005-2011, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ namespace torrent {
 
 class LIBTORRENT_EXPORT PollKQueue : public torrent::Poll {
 public:
-  typedef std::vector<uint32_t> Table;
+  typedef std::vector<std::pair<uint32_t, Event*> > Table;
 
   static const uint32_t flag_read  = (1 << 0);
   static const uint32_t flag_write = (1 << 1);
@@ -67,6 +67,9 @@ public:
   virtual void        open(torrent::Event* event);
   virtual void        close(torrent::Event* event);
 
+  // torrent::Event::get_fd() was closed outside of our control.
+  virtual void        closed(torrent::Event* event);
+
   // Functions for checking whetever the torrent::Event is listening to r/w/e?
   virtual bool        in_read(torrent::Event* event);
   virtual bool        in_write(torrent::Event* event);
@@ -83,22 +86,28 @@ public:
   virtual void        remove_error(torrent::Event* event);
 
 private:
-  PollKQueue(int fd, int maxEvents, int maxOpenSockets);
+  PollKQueue(int fd, int maxEvents, int maxOpenSockets) LIBTORRENT_NO_EXPORT;
 
-  inline uint32_t     event_mask(Event* e);
-  inline void         set_event_mask(Event* e, uint32_t m);
+  inline uint32_t     event_mask(Event* e) LIBTORRENT_NO_EXPORT;
+  inline void         set_event_mask(Event* e, uint32_t m) LIBTORRENT_NO_EXPORT;
 
-  void                modify(torrent::Event* event, unsigned short op, short mask);
+  int                 poll_select(int msec) LIBTORRENT_NO_EXPORT;
+
+  void                flush_events() LIBTORRENT_NO_EXPORT;
+  void                modify(torrent::Event* event, unsigned short op, short mask) LIBTORRENT_NO_EXPORT;
 
   int                 m_fd;
 
-  int                 m_maxEvents;
-  int                 m_waitingEvents;
-  int                 m_changedEvents;
+  unsigned int        m_maxEvents;
+  unsigned int        m_waitingEvents;
+  unsigned int        m_changedEvents;
 
   Table               m_table;
   struct kevent*      m_events;
   struct kevent*      m_changes;
+
+  // Work-around the stdin bug in MacOSX's kqueue implementation.
+  Event*              m_stdinEvent;
 };
 
 }
