@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2007, Jari Sundell
+// Copyright (C) 2005-2011, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -123,7 +123,8 @@ PollSelect::create(int maxOpenSockets) {
     SocketSet t4;
   };
 
-  block_type* block = new (rak::cacheline_allocator<>()) block_type;
+  rak::cacheline_allocator<Block*> cl_alloc;
+  block_type* block = new (cl_alloc) block_type;
 
   PollSelect* p = new (&block->t1) PollSelect;
 
