@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2007, Jari Sundell
+// Copyright (C) 2005-2011, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -48,29 +48,42 @@
 #include "torrent/http.h"
 #include "torrent/object_stream.h"
 #include "torrent/tracker_list.h"
+#include "torrent/utils/log.h"
 
 #include "tracker_http.h"
 
 #include "globals.h"
 #include "manager.h"
 
+#define LT_LOG_TRACKER(log_level, log_fmt, ...)                         \
+  lt_log_print_info(LOG_TRACKER_##log_level, m_parent->info(), "->tracker[%u]: " log_fmt, group(), __VA_ARGS__);
+
+namespace std { using namespace tr1; }
+
 namespace torrent {
 
-TrackerHttp::TrackerHttp(TrackerList* parent, const std::string& url) :
-  Tracker(parent, url),
+TrackerHttp::TrackerHttp(TrackerList* parent, const std::string& url, int flags) :
+  Tracker(parent, url, flags),
 
   m_get(Http::call_factory()),
   m_data(NULL) {
 
-  m_get->signal_done().connect(sigc::mem_fun(*this, &TrackerHttp::receive_done));
-  m_get->signal_failed().connect(sigc::mem_fun(*this, &TrackerHttp::receive_failed));
+  m_get->signal_done().push_back(std::bind(&TrackerHttp::receive_done, this));
+  m_get->signal_failed().push_back(std::bind(&TrackerHttp::receive_failed, this, std::placeholders::_1));
 
   // Haven't considered if this needs any stronger error detection,
   // can dropping the '?' be used for malicious purposes?
-  size_t delim = url.rfind('?');
+  size_t delim_options = url.rfind('?');
 
-  m_dropDeliminator = delim != std::string::npos &&
-    url.find('/', delim) == std::string::npos;
+  m_dropDeliminator = delim_options != std::string::npos &&
+    url.find('/', delim_options) == std::string::npos;
+
+  // Check the url if we can use scrape.
+  size_t delim_slash = url.rfind('/');
+
+  if (delim_slash != std::string::npos &&
+      url.find("/announce", delim_slash) == delim_slash)
+    m_flags |= flag_can_scrape;
 }
 
 TrackerHttp::~TrackerHttp() {
@@ -83,6 +96,17 @@ TrackerHttp::is_busy() const {
   return m_data != NULL;
 }
 
+void
+TrackerHttp::request_prefix(std::stringstream* stream, const std::string& url) {
+  char hash[61];
+
+  *rak::copy_escape_html(m_parent->info()->hash().begin(),
+                         m_parent->info()->hash().end(), hash) = '\0';
+  *stream << url
+          << (m_dropDeliminator ? '&' : '?')
+          << "info_hash=" << hash;
+}
+
 void
 TrackerHttp::send_state(int state) {
   close();
@@ -90,27 +114,26 @@ TrackerHttp::send_state(int state) {
   if (m_parent == NULL)
     throw internal_error("TrackerHttp::send_state(...) does not have a valid m_parent.");
 
+  m_latest_event = state;
+
   std::stringstream s;
   s.imbue(std::locale::classic());
 
-  char hash[61];
   char localId[61];
 
   DownloadInfo* info = m_parent->info();
 
-  *rak::copy_escape_html(info->hash().begin(), info->hash().end(), hash) = '\0';
+  request_prefix(&s, m_url);
+
   *rak::copy_escape_html(info->local_id().begin(), info->local_id().end(), localId) = '\0';
 
-  s << m_url
-    << (m_dropDeliminator ? '&' : '?')
-    << "info_hash=" << hash
-    << "&peer_id=" << localId;
+  s << "&peer_id=" << localId;
 
   if (m_parent->key())
     s << "&key=" << std::hex << std::setw(8) << std::setfill('0') << m_parent->key() << std::dec;
 
-  if (!m_trackerId.empty())
-    s << "&trackerid=" << rak::copy_escape_html(m_trackerId);
+  if (!m_tracker_id.empty())
+    s << "&trackerid=" << rak::copy_escape_html(m_tracker_id);
 
   const rak::socket_address* localAddress = rak::socket_address::cast_from(manager->connection_manager()->local_address());
 
@@ -147,7 +170,36 @@ TrackerHttp::send_state(int state) {
 
   m_data = new std::stringstream();
 
-  m_get->set_url(s.str());
+  std::string request_url = s.str();
+
+  LT_LOG_TRACKER(DEBUG, "Tracker HTTP Request ---\n%*s\n---", request_url.size(), request_url.c_str());
+
+  m_get->set_url(request_url);
+  m_get->set_stream(m_data);
+  m_get->set_timeout(2 * 60);
+
+  m_get->start();
+}
+
+void
+TrackerHttp::send_scrape() {
+  if (m_data != NULL)
+    return;
+
+  m_latest_event = EVENT_SCRAPE;
+
+  std::stringstream s;
+  s.imbue(std::locale::classic());
+
+  request_prefix(&s, scrape_url_from(m_url));
+
+  m_data = new std::stringstream();
+
+  std::string request_url = s.str();
+
+  LT_LOG_TRACKER(DEBUG, "Tracker HTTP Scrape ---\n%*s\n---", request_url.size(), request_url.c_str());
+  
+  m_get->set_url(request_url);
   m_get->set_stream(m_data);
   m_get->set_timeout(2 * 60);
 
@@ -176,12 +228,9 @@ TrackerHttp::receive_done() {
   if (m_data == NULL)
     throw internal_error("TrackerHttp::receive_done() called on an invalid object");
 
-  DownloadInfo* info = m_parent->info();
-
-  if (!info->signal_tracker_dump().empty()) {
+  if (lt_log_is_valid(LOG_TRACKER_DEBUG)) {
     std::string dump = m_data->str();
-
-    info->signal_tracker_dump().emit(m_get->url(), dump.c_str(), dump.size());
+    LT_LOG_TRACKER(DEBUG, "Tracker HTTP receive done ---\n%*s\n---", dump.size(), dump.c_str());
   }
 
   Object b;
@@ -200,34 +249,57 @@ TrackerHttp::receive_done() {
 			  std::string("failure reason not a string"))
 			 + "\"");
 
-  if (b.has_key_value("interval"))
-    set_normal_interval(b.get_key_value("interval"));
-  
-  if (b.has_key_value("min interval"))
-    set_min_interval(b.get_key_value("min interval"));
+  if (m_latest_event == EVENT_SCRAPE)
+    process_scrape(b);
+  else
+    process_success(b);
+}
 
-  if (b.has_key_string("tracker id"))
-    m_trackerId = b.get_key_string("tracker id");
-
-  if (b.has_key_value("complete") && b.has_key_value("incomplete")) {
-    m_scrapeComplete   = std::max<int64_t>(b.get_key_value("complete"), 0);
-    m_scrapeIncomplete = std::max<int64_t>(b.get_key_value("incomplete"), 0);
-    m_scrapeTimeLast   = rak::timer::current().seconds();
+void
+TrackerHttp::receive_failed(std::string msg) {
+  if (lt_log_is_valid(LOG_TRACKER_DEBUG)) {
+    std::string dump = m_data->str();
+    LT_LOG_TRACKER(DEBUG, "Tracker HTTP receive failed ---\n%*s\n---", dump.size(), dump.c_str());
   }
 
-  if (b.has_key_value("downloaded"))
-    m_scrapeDownloaded = std::max<int64_t>(b.get_key_value("downloaded"), 0);
+  close();
+
+  if (m_latest_event == EVENT_SCRAPE)
+    m_parent->receive_scrape_failed(this, msg);
+  else
+    m_parent->receive_failed(this, msg);
+}
+
+void
+TrackerHttp::process_success(const Object& object) {
+  if (object.has_key_value("interval"))
+    set_normal_interval(object.get_key_value("interval"));
+  
+  if (object.has_key_value("min interval"))
+    set_min_interval(object.get_key_value("min interval"));
+
+  if (object.has_key_string("tracker id"))
+    m_tracker_id = object.get_key_string("tracker id");
+
+  if (object.has_key_value("complete") && object.has_key_value("incomplete")) {
+    m_scrape_complete = std::max<int64_t>(object.get_key_value("complete"), 0);
+    m_scrape_incomplete = std::max<int64_t>(object.get_key_value("incomplete"), 0);
+    m_scrape_time_last = cachedTime.seconds();
+  }
+
+  if (object.has_key_value("downloaded"))
+    m_scrape_downloaded = std::max<int64_t>(object.get_key_value("downloaded"), 0);
 
   AddressList l;
 
   try {
     // Due to some trackers sending the wrong type when no peers are
     // available, don't bork on it.
-    if (b.get_key("peers").is_string())
-      l.parse_address_compact(b.get_key_string("peers"));
+    if (object.get_key("peers").is_string())
+      l.parse_address_compact(object.get_key_string("peers"));
 
-    else if (b.get_key("peers").is_list())
-      l.parse_address_normal(b.get_key_list("peers"));
+    else if (object.get_key("peers").is_list())
+      l.parse_address_normal(object.get_key_list("peers"));
 
   } catch (bencode_error& e) {
     return receive_failed(e.what());
@@ -238,10 +310,32 @@ TrackerHttp::receive_done() {
 }
 
 void
-TrackerHttp::receive_failed(std::string msg) {
-  // Does the order matter?
+TrackerHttp::process_scrape(const Object& object) {
+  if (!object.has_key_map("files"))
+    return receive_failed("Tracker scrape does not have files entry.");
+
+  // Add better validation here...
+  const Object& files = object.get_key("files");
+
+  if (!files.has_key_map(m_parent->info()->hash().str()))
+    return receive_failed("Tracker scrape replay did not contain infohash.");
+
+  const Object& stats = files.get_key(m_parent->info()->hash().str());
+
+  if (stats.has_key_value("complete"))
+    m_scrape_complete = std::max<int64_t>(stats.get_key_value("complete"), 0);
+
+  if (stats.has_key_value("incomplete"))
+    m_scrape_incomplete = std::max<int64_t>(stats.get_key_value("incomplete"), 0);
+
+  if (stats.has_key_value("downloaded"))
+    m_scrape_downloaded = std::max<int64_t>(stats.get_key_value("downloaded"), 0);
+
+  LT_LOG_TRACKER(INFO, "Tracker scrape for %u torrents: complete:%u incomplete:%u downloaded:%u.",
+                 files.as_map().size(), m_scrape_complete, m_scrape_incomplete, m_scrape_downloaded);
+
   close();
-  m_parent->receive_failed(this, msg);
+  m_parent->receive_scrape_success(this);
 }
 
 }
